/*
*	File:		MelodyExtraction.cpp
*	
*	Version:	1.0
* 
*	Created:	10/08/10
*	
*	Copyright:  Copyright  2010 Angel Noguera, All Rights Reserved
* 
*	Disclaimer:	IMPORTANT:  This Apple software is supplied to you by Apple Computer, Inc. ("Apple") in 
*				consideration of your agreement to the following terms, and your use, installation, modification 
*				or redistribution of this Apple software constitutes acceptance of these terms.  If you do 
*				not agree with these terms, please do not use, install, modify or redistribute this Apple 
*				software.
*
*				In consideration of your agreement to abide by the following terms, and subject to these terms, 
*				Apple grants you a personal, non-exclusive license, under Apple's copyrights in this 
*				original Apple software (the "Apple Software"), to use, reproduce, modify and redistribute the 
*				Apple Software, with or without modifications, in source and/or binary forms; provided that if you 
*				redistribute the Apple Software in its entirety and without modifications, you must retain this 
*				notice and the following text and disclaimers in all such redistributions of the Apple Software. 
*				Neither the name, trademarks, service marks or logos of Apple Computer, Inc. may be used to 
*				endorse or promote products derived from the Apple Software without specific prior written 
*				permission from Apple.  Except as expressly stated in this notice, no other rights or 
*				licenses, express or implied, are granted by Apple herein, including but not limited to any 
*				patent rights that may be infringed by your derivative works or by other works in which the 
*				Apple Software may be incorporated.
*
*				The Apple Software is provided by Apple on an "AS IS" basis.  APPLE MAKES NO WARRANTIES, EXPRESS OR 
*				IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY 
*				AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND OPERATION ALONE 
*				OR IN COMBINATION WITH YOUR PRODUCTS.
*
*				IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL 
*				DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*				OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, 
*				REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER 
*				UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN 
*				IF APPLE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/
/*=============================================================================
	MelodyExtraction.cpp
	
=============================================================================*/
#include "MelodyExtraction.h"
#include "myfft.h"


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

COMPONENT_ENTRY(MelodyExtraction)


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//	MelodyExtraction::MelodyExtraction
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
MelodyExtraction::MelodyExtraction(AudioUnit component)
	: AUEffectBase(component)
{
	CreateElements();
	Globals()->UseIndexedParameters(kNumberOfParameters);
SetParameter ( // 1 
 kParameter_Frec_Min, 
 kDefaultValue_Melody_Frec_Min 
); 
SetParameter ( // 2 
kParameter_Frec_Max, 
kDefaultValue_Melody_Frec_Max 
); 

SetParameter ( // 3 
kParameter_Umbral, 
kDefaultValue_Melody_Umbral
);

SetParameter ( // 3 
kParameter_Sol, 
kDefaultValue_Melody_Sol
);

SetAFactoryPresetAsCurrent (
kPresets [kPreset_Default] 
);
        
#if AU_DEBUG_DISPATCHER
	mDebugDispatcher = new AUDebugDispatcher (this);
#endif
	
}


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//	MelodyExtraction::GetParameterValueStrings
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
ComponentResult		MelodyExtraction::GetParameterValueStrings(AudioUnitScope		inScope,
                                                                AudioUnitParameterID	inParameterID,
                                                                CFArrayRef *		outStrings)
{
        
    return kAudioUnitErr_InvalidProperty;
}



//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//	MelodyExtraction::GetParameterInfo
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
ComponentResult		MelodyExtraction::GetParameterInfo(AudioUnitScope		inScope,
                                                        AudioUnitParameterID	inParameterID,
                                                        AudioUnitParameterInfo	&outParameterInfo )
{
	ComponentResult result = noErr;

	outParameterInfo.flags = 	kAudioUnitParameterFlag_IsWritable
						|		kAudioUnitParameterFlag_IsReadable;
    
    if (inScope == kAudioUnitScope_Global) {
        switch(inParameterID)
        {
            case kParameter_Frec_Min:  
					
				AUBase::FillInParameterName (outParameterInfo, kParamName_Melody_Frec_Min, false ); 
					
					outParameterInfo.unit =  
						kAudioUnitParameterUnit_Hertz; 
					outParameterInfo.minValue = 
						kMinimumValue_Melody_Frec_Min; 
					outParameterInfo.maxValue =  
						kMaximumValue_Melody_Frec_Min; 
					outParameterInfo.defaultValue =  
					kDefaultValue_Melody_Frec_Min; 
					outParameterInfo.flags  
						|= kAudioUnitParameterFlag_DisplayLogarithmic; 
				break; 
				
				case kParameter_Frec_Max:  
					
				AUBase::FillInParameterName (outParameterInfo, kParamName_Melody_Frec_Max, false ); 
					
					outParameterInfo.unit =
						kAudioUnitParameterUnit_Hertz; 
					outParameterInfo.minValue =  
						kMinimumValue_Melody_Frec_Max; 
					outParameterInfo.maxValue = 
						kMaximumValue_Melody_Frec_Max; 
					outParameterInfo.defaultValue = 
						kDefaultValue_Melody_Frec_Max; 
					outParameterInfo.flags  
						|= kAudioUnitParameterFlag_DisplayLogarithmic; 
				break; 
				
				case kParameter_Umbral:  
					
				AUBase::FillInParameterName (outParameterInfo, kParamName_Melody_Umbral, false ); 
					
					outParameterInfo.unit =
						
						kAudioUnitParameterUnit_Percent; 
					outParameterInfo.minValue =  
						kMinimumValue_Melody_Umbral; 
					outParameterInfo.maxValue = 
						kMaximumValue_Melody_Umbral; 
					outParameterInfo.defaultValue = 
						kDefaultValue_Melody_Umbral; 
				
				break; 
				
				case kParameter_Sol:  
					
				AUBase::FillInParameterName (outParameterInfo, kParamName_Melody_Sol, false ); 
					
					outParameterInfo.unit =
						
						kAudioUnitParameterUnit_Percent; 
					outParameterInfo.minValue =  
						kMinimumValue_Melody_Sol; 
					outParameterInfo.maxValue = 
						kMaximumValue_Melody_Sol; 
					outParameterInfo.defaultValue = 
						kDefaultValue_Melody_Sol; 
				break; 

            default:
                result = kAudioUnitErr_InvalidParameter;
                break;
            }
	} else {
        result = kAudioUnitErr_InvalidParameter;
    }
    


	return result;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//	MelodyExtraction::GetPropertyInfo
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
ComponentResult		MelodyExtraction::GetPropertyInfo (AudioUnitPropertyID	inID,
                                                        AudioUnitScope		inScope,
                                                        AudioUnitElement	inElement,
                                                        UInt32 &		outDataSize,
                                                        Boolean &		outWritable)
{
	return AUEffectBase::GetPropertyInfo (inID, inScope, inElement, outDataSize, outWritable);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//	MelodyExtraction::GetProperty
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
ComponentResult		MelodyExtraction::GetProperty(	AudioUnitPropertyID inID,
                                                        AudioUnitScope 		inScope,
                                                        AudioUnitElement 	inElement,
                                                        void *			outData )
{
	return AUEffectBase::GetProperty (inID, inScope, inElement, outData);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//	MelodyExtraction::GetPresets
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#pragma mark ____Factory Presets 
ComponentResult MelodyExtraction::GetPresets ( // 1 
CFArrayRef *outData 
) const { 
if (outData == NULL) return noErr; // 2 
CFMutableArrayRef presetsArray = CFArrayCreateMutable ( // 3 
NULL, 
kNumberPresets, 
NULL 
); 
for (int i = 0; i < kNumberPresets; ++i) { // 4 
CFArrayAppendValue ( 
presetsArray, 
&kPresets [i] 
); 
} 
*outData = (CFArrayRef) presetsArray; // 5 
return noErr; 
} 

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//	MelodyExtraction::NewFactoryPresetSet
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

OSStatus MelodyExtraction::NewFactoryPresetSet (
	const AUPreset &inNewFactoryPreset 
) { 
	SInt32 chosenPreset = inNewFactoryPreset.presetNumber; 
	
	if ( // 3 
		chosenPreset == kPreset_Saxo || 
		chosenPreset == kPreset_Trompeta ||
		chosenPreset == kPreset_Viola
	) { 
		for (int i = 0; i < kNumberPresets; ++i) { // 4 
			if (chosenPreset == kPresets[i].presetNumber) { 
				switch (chosenPreset) {  

					case kPreset_Saxo: 
						
						SetParameter ( 
							kParameter_Frec_Min, 
							kParameter_Preset_Frec_Min_Saxo 
						); 
						
						SetParameter ( 
							kParameter_Frec_Max, 
							kParameter_Preset_Frec_Max_Saxo 
						); 
					
						SetParameter ( 
							kParameter_Umbral, 
							kParameter_Preset_Umbral_Saxo
						); 
						
						SetParameter ( 
							kParameter_Sol, 
							kParameter_Preset_Sol_Saxo
						); 

				
					break; 
					
					case kPreset_Trompeta: 
						
						SetParameter ( 
							kParameter_Frec_Min, 
							kParameter_Preset_Frec_Min_Trompeta 
						); 
						
						SetParameter ( 
							kParameter_Frec_Max, 
							kParameter_Preset_Frec_Max_Trompeta  
						); 
						SetParameter ( 
							kParameter_Umbral, 
							kParameter_Preset_Umbral_Trompeta
						); 
						
						SetParameter ( 
							kParameter_Sol, 
							kParameter_Preset_Sol_Trompeta
						); 

						
					break; 
					
					case kPreset_Viola: 
						
						SetParameter ( 
							kParameter_Frec_Min, 
							kParameter_Preset_Frec_Min_Viola 
						); 
						
						SetParameter ( 
							kParameter_Frec_Max, 
							kParameter_Preset_Frec_Max_Viola  
						); 
					
						SetParameter ( 
							kParameter_Umbral, 
							kParameter_Preset_Umbral_Viola
						); 
						
						SetParameter ( 
							kParameter_Sol, 
							kParameter_Preset_Sol_Viola
						); 

					break; 
				} 
				SetAFactoryPresetAsCurrent ( // 11 
					kPresets [i] 
				); 
				return noErr; // 12 
			} 
		} 
	} 
	return kAudioUnitErr_InvalidProperty; // 13 
} 
#pragma mark ____MelodyExtractionEffectKernel


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//	MelodyExtraction::MelodyExtractionKernel::Reset()
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void		MelodyExtraction::MelodyExtractionKernel::Reset()
{

	guard = 0;
}

int	MelodyExtraction::MelodyExtractionKernel::FreqToPos(Float32 freq, Float32 freqRes)
{
	
	int pos;
	
	pos = freq / freqRes;
	
	if (pos < 1)
	{
		pos = 1;
	}
	
	return pos;
}

Float32	MelodyExtraction::MelodyExtractionKernel::PosToFreq(int pos, Float32 freqRes)
{
	
	Float32 freq;
	
	freq = (Float32)pos * freqRes;
	
	if (freq < 20)
	{
		freq = 20;
	}
	
	return freq;
}

Float32	MelodyExtraction::MelodyExtractionKernel::RMS(Float32 array[], int size)
{
		int i;
		Float32 sum = 0;
		
		for(i = 0 ; i < size ; i++)
		{
			sum = sum + array[i]*array[i];
		}
		
		sum = sqrt(sum/size);
		return sum;
}

void MelodyExtraction::MelodyExtractionKernel::HPS(int winSize, int parNum)
{
	int i;
	Float32 reducedSpectrum[winSize];
	
	for(i = 0; i < winSize/(parNum) ; i++)
	{
		reducedSpectrum[i] = inSpectrum.mod[i*parNum];
	}
	
	for(i = winSize/(parNum); i < winSize ; i++)
	{
		reducedSpectrum[i] = 0;
	}
	
	for(i = 0 ; i < winSize ; i++)
	{
		averageSpectrum[i] = averageSpectrum[i]*reducedSpectrum[i];
	}
}

int	MelodyExtraction::MelodyExtractionKernel::MaximumSearch(int begOfRange, int endOfRange)
{
	int maxPos = - 1, i, j = 0;
	Float32 maxVal = 0;
	
	if(begOfRange > endOfRange)
	{
		begOfRange = endOfRange -1;
	}
	
	for(i = begOfRange ; i < endOfRange - 1; i++)
	{
		if(averageSpectrum[i-1] < averageSpectrum[i] && averageSpectrum[i+1] < averageSpectrum[i])
		{
			peaks[j] = i;
			j++;
		}
	}
	
	for(i = 0 ; i < j ; i++)
	{
		if(averageSpectrum[peaks[i]] > maxVal)
		{
			maxVal = averageSpectrum[peaks[i]];
			maxPos = peaks[i];
		}
	}
	
	return maxPos;
}


void	MelodyExtraction::MelodyExtractionKernel::Synthesis(UInt32 nSample, int partInd)
{
	Float32 sinusoidal[nSample], amp, ampmed, x = 0, nu = 0, tau = 0, faseinstantanea;
	int n;

	ampmed = (part[partInd].mod - prePart[partInd].mod)/((double)nSample);
	
	x = (1.0/(2.0 * M_PI)) * ((prePart[partInd].phas + prePart[partInd].omega * (double)nSample - part[partInd].phas) + ((double)nSample/2.0) * (part[partInd].omega-prePart[partInd].omega));
	nu = (3.0/((double)nSample*(double)nSample)) * (part[partInd].phas - prePart[partInd].phas - prePart[partInd].omega * (double)nSample + 2.0 * M_PI * round(x)) - (1.0/(double)nSample) * (part[partInd].omega - prePart[partInd].omega);
	tau = -(2.0/((double)nSample*(double)nSample*(double)nSample)) * (part[partInd].phas - prePart[partInd].phas - prePart[partInd].omega * (double)nSample + 2.0 * M_PI * round(x)) - (1.0/(double)nSample*(double)nSample) * (part[partInd].omega - prePart[partInd].omega);
	
	for(n = 0 ; n < (int)nSample ; n++)
	{
		amp = prePart[partInd].mod + (ampmed * (Float32)n);
		faseinstantanea = prePart[partInd].phas + prePart[partInd].omega * (Float32)n + nu * ((Float32)n * (Float32)n) + tau * ((Float32)n * (Float32)n * (Float32)n);
		sinusoidal[n] = amp * cos(faseinstantanea/GetSampleRate());
		synthSignal[n] = synthSignal[n] + sinusoidal[n];	
	}
	
	return;	
}

int MelodyExtraction::MelodyExtractionKernel::SelectPeakCloseTo(Float32 spectrum[], int tamwindow, int sr, Float32 centerfreq, int tolerance) 
{		
	int nearest=-1;
	double maxval;
	double lowlimit, highlimit;

	if (centerfreq < sr/2) 
	{
		lowlimit = centerfreq - tolerance;
		highlimit = centerfreq + tolerance;

		maxval=0;
		
		int i;
		for (i = 0; i < tamwindow && (Float32)i * ((Float32)sr/(Float32)tamwindow) <= lowlimit; i++);
		while (i < tamwindow && (Float32)i * ((Float32)sr/(Float32)tamwindow) <= centerfreq)
		{
			double bandfactor = ( (Float32)i * ( (Float32)sr / (Float32)tamwindow ) - lowlimit) / ( centerfreq - lowlimit);
			double val = spectrum[i] * bandfactor;
			if (val > maxval) 
			{
				maxval = val;
				nearest = i;
			
			}
			i++;	
		}
		
		while (i < tamwindow && ((Float32)i*((Float32)sr/(Float32)tamwindow)<=highlimit))
		{
			double bandfactor = ((highlimit - centerfreq) - ((Float32)i * ( (Float32)sr / (Float32)tamwindow ) - centerfreq)) / (highlimit - centerfreq);

			double val = spectrum[i] * bandfactor; 
			if (val > maxval) 
			{
				maxval = val;
				nearest = i;
			
			}
		        i++;
		}
	}
	return nearest;
}

void MelodyExtraction::MelodyExtractionKernel::Overlap(int size, Float32 coef)
{
	int i, cue = size*coef/100;
	
	for(i = 0 ; i < cue ; i++)
	{
		overlapSignal[i] = preSynthSignal[size - cue + i];
	}
	
	for(i = cue ; i < size ; i++)
	{
		overlapSignal[i] = synthSignal[i - cue];
	}
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//	MelodyExtraction::MelodyExtractionKernel::Process
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void		MelodyExtraction::MelodyExtractionKernel::Process(	const Float32 	*inSourceP,
                                                    Float32		 	*inDestP,
                                                    UInt32 			inFramesToProcess,
                                                    UInt32			inNumChannels, // for version 2 AudioUnits inNumChannels is always 1
                                                    bool			&ioSilence )
{

	if(!ioSilence && (inFramesToProcess == 4096 || inFramesToProcess == 2048 || inFramesToProcess == 1024))
	{
		UInt32 nSampleFrames = inFramesToProcess; 
		const Float32 *sourceP = inSourceP; 
		Float32 *destP = inDestP;
		int i,j;
		
		Float32 maxFreq = GetParameter (kParameter_Frec_Max), minFreq = GetParameter (kParameter_Frec_Min), threshold = GetParameter (kParameter_Umbral), overlapCoef= GetParameter (kParameter_Sol);
		int sampleRate = GetSampleRate();
		
		int zeroFrames = 0, anaWinSize = nSampleFrames * (1 + zeroFrames);
		Float32 anaFreqRes = (Float32)sampleRate/(Float32)anaWinSize;
		int anaMaxFreqPos = FreqToPos(maxFreq, anaFreqRes) , anaMinFreqPos = FreqToPos(minFreq, anaFreqRes);
		int nHPSpartials = 5;
		int numOfPart, guard2 = 0;
		Float32 inRMS = 0;
		for(i = 0; i < inFramesToProcess ; i++)
		{
			if(inSourceP[i] > 1 || inSourceP[i] < -1)
			{
				guard2 = 1;
			}
		}
		
		for(i = 0 ; i < (int)nSampleFrames ; i++)
		{
			in[i] = inSourceP[i];
			synthSignal[i] = 0;
			peaks[i] = 0;
		}
		
		inRMS = RMS(in, nSampleFrames);
		
		//******************ANLISIS************************/
		if(inRMS > threshold / 100 && guard2 == 0)
		{
			Hanning(inSpectrum.win,nSampleFrames);
			
			if(anaWinSize > nSampleFrames)
			{
				for(i = nSampleFrames ; i < anaWinSize ; i++ )
				{
					inSpectrum.win[i] = 1;
					in[i] = 0;
				}
			}
			inSpectrum.size = anaWinSize;
			fourier_spectrum(in, inSpectrum.mod, inSpectrum.pha, anaWinSize, inSpectrum.size, inSpectrum.win, anaWinSize);
			
			for(i = 0 ; i < anaWinSize/2 ; i++)
			{
				averageSpectrum[i] = inSpectrum.mod[i];
			}
			
			for(i = 2; i <= nHPSpartials ; i++)
			{
				HPS(anaWinSize/2, i);
			}
			
			for(j = 1 ; j <= nHPSpartials ; j++)
			{
				for(i = 0 ; i < anaWinSize/2 ; i++)
				{
					averageSpectrum[i] = averageSpectrum[i] / anaFreqRes;
				}
			}
			
			averageSpectrum[i] = averageSpectrum[i] / nHPSpartials;
			
			part[0].pos = MaximumSearch(anaMinFreqPos, anaMaxFreqPos);
			
			//---------------------------SNTESIS------------------------
			
			numOfPart = ((anaWinSize/2)/part[0].pos) - 1;

			if(numOfPart > 15)
			{
				numOfPart = 15;
			}
			
			if(numOfPart < 0)
			{
				numOfPart = 0;
			}
			
			for(i = 0 ; i < numOfPart ; i++)
			{
				if(i != 0)
				{
					part[i].pos = SelectPeakCloseTo(inSpectrum.mod, anaWinSize, sampleRate, PosToFreq((1+i)*part[0].pos, anaFreqRes), 4*anaFreqRes);
				}
				
				if(part[i].pos == -1)
				{
					part[i].mod = 0;
					part[i].phas = 0;
					part[i].omega = 0;
				}else
				{
					part[i].mod = inSpectrum.mod[part[i].pos];
					part[i].phas = inSpectrum.pha[part[i].pos];
					part[i].omega = 2.0 * M_PI * PosToFreq(part[i].pos, anaFreqRes);
				}
					
				if(guard == 0)
				{
					prePart[i] = part[i];
					guard = 1;
				}
				
				if(part[i].pos != -1)
				{
					Synthesis(nSampleFrames, i);
				}
				
			}
			
			Overlap(nSampleFrames, overlapCoef);
			
			for(i = 0 ; i < numOfPart ; i++)
			{
				prePart[i] = part[i];
			}
			for(i = 0; i < (int)nSampleFrames; i++)
			{
				preSynthSignal[i] = synthSignal[i];
				destP[i] = overlapSignal[i]/(nSampleFrames/4);
			}
		}else
		{
			for(i = 0; i < (int)nSampleFrames; i++)
			{
				destP[i] = 0;
				preSynthSignal[i] = 0;
			}
		}
		
	}
}

